﻿using System;
using System.Configuration;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Runtime.Remoting;
using System.ServiceProcess;
using System.Text;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.Security;
using DevExpress.Persistent.Base;
using DevExpress.Persistent.BaseImpl.PermissionPolicy;
using DevExpress.ExpressApp.Security.ClientServer;
using DevExpress.ExpressApp.Security.ClientServer.Remoting;
using System.ServiceModel;
using DevExpress.ExpressApp.Security.ClientServer.Wcf;
using DevExpress.ExpressApp.Xpo;
using DevExpress.ExpressApp.MiddleTier;
using DevExpress.ExpressApp.Security.Strategy;

namespace Solution1.ApplicationServer {
    public partial class ApplicationServerService : System.ServiceProcess.ServiceBase {
        private ServiceHost serviceHost;
        private void serverApplication_DatabaseVersionMismatch(object sender, DatabaseVersionMismatchEventArgs e) {
            e.Updater.Update();
            e.Handled = true;
        }
        protected override void OnStart(string[] args) {
            var connectionString = ConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString;
            ValueManager.ValueManagerType = typeof(MultiThreadValueManager<>).GetGenericTypeDefinition();

            var serverApplication = new ServerApplication {
                ApplicationName = @"Solution1", CheckCompatibilityType = CheckCompatibilityType.DatabaseSchema
            };
#if DEBUG
            if(Debugger.IsAttached && serverApplication.CheckCompatibilityType == CheckCompatibilityType.DatabaseSchema) {
                serverApplication.DatabaseUpdateMode = DatabaseUpdateMode.UpdateDatabaseAlways;
            }
#endif

            serverApplication.Modules.BeginInit();
            serverApplication.Modules.Add(new SecurityModule());
            serverApplication.Modules.Add(new Module.Solution1Module());
            serverApplication.Modules.Add(new Module.Win.Solution1WindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.AuditTrail.AuditTrailModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Objects.BusinessClassLibraryCustomizationModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Chart.ChartModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Chart.Win.ChartWindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.CloneObject.CloneObjectModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.ConditionalAppearance.ConditionalAppearanceModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Dashboards.DashboardsModule() { DashboardDataType = typeof(DevExpress.Persistent.BaseImpl.DashboardData) });
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Dashboards.Win.DashboardsWindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.FileAttachments.Win.FileAttachmentsWindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Kpi.KpiModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Notifications.NotificationsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Notifications.Win.NotificationsWindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Office.Win.OfficeWindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.PivotChart.PivotChartModuleBase());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.PivotChart.Win.PivotChartWindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.PivotGrid.PivotGridModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.PivotGrid.Win.PivotGridWindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.ReportsV2.ReportsModuleV2() { ReportDataType = typeof(DevExpress.Persistent.BaseImpl.ReportDataV2) });
            serverApplication.Modules.Add(new DevExpress.ExpressApp.ReportsV2.Win.ReportsWindowsFormsModuleV2());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.TreeListEditors.TreeListEditorsModuleBase());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.TreeListEditors.Win.TreeListEditorsWindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Validation.ValidationModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Validation.Win.ValidationWindowsFormsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.ViewVariantsModule.ViewVariantsModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Workflow.WorkflowModule());
            serverApplication.Modules.Add(new DevExpress.ExpressApp.Workflow.Win.WorkflowWindowsFormsModule());
            // ---
            serverApplication.Modules.Add(new Xafari.XafariModule());
            serverApplication.Modules.Add(new Xafari.BC.XafariBCModule());
            serverApplication.Modules.Add(new Xafari.DockPanels.XafariDockPanelsModule());
            serverApplication.Modules.Add(new Xafari.Editors.XafariEditorsModule());
            serverApplication.Modules.Add(new Xafari.Filters.XafariFiltersModule());
            serverApplication.Modules.Add(new Xafari.Security.XafariSecurityModule());
            serverApplication.Modules.Add(new Xafari.Security.DC.XafariSecurityDCModule());
            serverApplication.Modules.Add(new Xafari.SmartDesign.XafariSmartDesignModule());
            serverApplication.Modules.Add(new Xafari.BC.DC.XafariBCDCModule());
            serverApplication.Modules.Add(new Xafari.BC.Numerators.XafariBCNumeratorsModule());
            serverApplication.Modules.Add(new Xafari.BC.Numerators.DC.XafariBCNumeratorsDCModule());
            serverApplication.Modules.Add(new Xafari.BC.Settings.XafariBCSettingsModule());
            serverApplication.Modules.Add(new Xafari.ManagedOperations.XafariManagedOperationsModule());
            serverApplication.Modules.Add(new Xafari.BC.BusinessOperations.Commands.XafariBusinessOperationsCommandsModule());
            serverApplication.Modules.Add(new Xafari.BC.BusinessOperations.XafariBusinessOperationsModule());
            serverApplication.Modules.Add(new Xafari.XF.XafariXFModule());
            serverApplication.Modules.EndInit();
            serverApplication.DatabaseVersionMismatch += serverApplication_DatabaseVersionMismatch;
            serverApplication.CreateCustomObjectSpaceProvider += (s, e) => {
                e.ObjectSpaceProviders.Add(new XPObjectSpaceProvider(e.ConnectionString, e.Connection));
                e.ObjectSpaceProviders.Add(new NonPersistentObjectSpaceProvider(serverApplication.TypesInfo, null));
            };

            serverApplication.ConnectionString = connectionString;

            serverApplication.Setup();
            serverApplication.CheckCompatibility();
            serverApplication.Dispose();

            IDataServerSecurity DataServerSecurityProvider() {
                var security = new SecurityStrategyComplex(typeof(PermissionPolicyUser), typeof(PermissionPolicyRole),
                    new AuthenticationStandard()) {SupportNavigationPermissionsForTypes = false};
                return security;
            }

            serviceHost = new WcfXafServiceHost(connectionString, DataServerSecurityProvider);
            serviceHost.AddServiceEndpoint(typeof(IWcfXafDataServer), WcfDataServerHelper.CreateNetTcpBinding(), "net.tcp://127.0.0.1:1451/DataServer");
            serviceHost.Open();
        }
        protected override void OnStop() {
            serviceHost.Close();
        }
        static ApplicationServerService() {
            PasswordCryptographer.EnableRfc2898 = true;
            PasswordCryptographer.SupportLegacySha512 = false;
        }
        public ApplicationServerService() {
            InitializeComponent();
        }
    }
}
