﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.ServiceModel;
using System.ServiceProcess;
using System.Text;
using System.Threading;

namespace Solution1.ApplicationServer
{
    internal static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        private static void Main() {
            var servicesToRun = new ServiceBase[] { new ApplicationServerService() };

            if (Environment.UserInteractive) {
                RunInteractive(servicesToRun);
            }
            else {
                ServiceBase.Run(servicesToRun);
            }
        }

        private static void RunInteractive(ServiceBase[] servicesToRun)
        {
            Console.WriteLine(@"Services running in interactive mode.");
            Console.WriteLine();

            var onStartMethod = typeof(ServiceBase).GetMethod("OnStart",
                BindingFlags.Instance | BindingFlags.NonPublic);

            foreach (var service in servicesToRun)
            {
                Console.Write(@"Starting {0}...", service.ServiceName);
                onStartMethod?.Invoke(service, new object[] { new string[] { } });
                Console.Write(@"Started");
            }

            Console.WriteLine();
            Console.WriteLine(@"Press any key to stop the services and end the process...");
            Console.ReadKey();
            Console.WriteLine();

            var onStopMethod = typeof(ServiceBase).GetMethod("OnStop",
                BindingFlags.Instance | BindingFlags.NonPublic);

            foreach (var service in servicesToRun)
            {
                Console.Write(@"Stopping {0}...", service.ServiceName);
                onStopMethod?.Invoke(service, null);
                Console.WriteLine(@"Stopped");
            }

            Console.WriteLine(@"All services stopped.");
            Thread.Sleep(1000);
        }

        public static System.ServiceModel.Channels.Binding CreateDefaultBinding()
        {
            var binding = new NetTcpBinding {
                MaxReceivedMessageSize = int.MaxValue,
                MaxBufferPoolSize = int.MaxValue,
                ReceiveTimeout = TimeSpan.FromHours(24),
                ReaderQuotas =
                {
                    MaxArrayLength = int.MaxValue,
                    MaxBytesPerRead = int.MaxValue,
                    MaxStringContentLength = int.MaxValue,
                    MaxDepth = int.MaxValue,
                    MaxNameTableCharCount = int.MaxValue
                }
            };
            return binding;
        }
    }
}
